import {LitElement, html, css} from 'lit-element';
import {customElement, property, query} from 'lit/decorators.js';

const KeyCodes = {
  ENTER: 'Enter',
  NUMPAD_ENTER: 'NumpadEnter'
};

const TodoInputEventType = {
  SUBMIT_TODO: 'submit-todo'
};

@customElement('todo-input')
export class TodoInputElement extends LitElement {
  static styles = css`
    :host {
      display: block;
    }
    input[type="text"] {
      width: 100%;
      padding: 16px;
      border: none;
      background: rgba(0, 0, 0, 0.003);
      box-shadow: inset 0 -2px 1px rgb(0 0 0 / 3%);
      font-size: 14px;
    }
  `;

  @query('input', true)
  _input!: HTMLInputElement;

  @property({type: String})
  description = '';

  @property({type: String})
  placeholder = '';

  @property({type: Boolean})
  newTodo = false

  render() {
    return html`
      <input type="text"
        value="${this.description || ''}"
        placeholder="${this.placeholder || ''}"
        @keyup="${this._onkeyup}">
    `;
  }

  private _onkeyup(event: KeyboardEvent) {
    if (event.code === KeyCodes.ENTER || event.code === KeyCodes.NUMPAD_ENTER) {
      this._submitTodo(this._input.value);
    }
  }

  private _submitTodo(description: string) {
    this.dispatchEvent(new CustomEvent(TodoInputEventType.SUBMIT_TODO, {
      bubbles: true,
      composed: true,
      detail: description
    }));
    this._input.value = '';
  }

  focus() {
    this._input?.focus();
  }
}


declare global {
  interface HTMLElementTagNameMap {
    'todo-input': TodoInputElement;
  }
}
