import {LitElement, html, css} from 'lit-element';
import {customElement, property} from 'lit/decorators.js';

import TodoModel from '../models/todo-model';
import './todo-input';

@customElement('todo-item')
export class TodoItemElement extends LitElement {
  static styles = css`
    :host {
      display: block;
      margin: 7px 0;
      font-family: sans-serif;
    }
    .todo-item {
      display: flex;
      flex-direction: row;
    }
    button.remove-button {
      visibility: hidden;
    }
    .todo-item:hover > button.remove-button,
    .todo-item input:focus ~ button.remove-button,
    .todo-item button.remove-button:focus {
      visibility: visible;
    }
    .description {
      display: flex;
      flex-direction: column;
      font-size: 14px;
      font-weight: 400;
    }
    .description > span {
      padding: 16px;
    }
    .created {
      font-size: 10px;
    }

  `;

  @property({type: Object})
  todo: TodoModel = new TodoModel('');

  @property({type: Boolean})
  completed = false;

  @property({type: Boolean})
  editing = false;

  render() {
    return html`
      <div class="todo-item">
        <input type="checkbox"
          .checked="${this.completed}"
          name="Completed"
          @change="${this._checkedChanged}">
        <div class="description">
          ${this.editing ? html`
            <todo-input
              .description="${this.todo.description}"
              @submit-todo="${this._updateTodo}"></todo-input>
          ` : html`
            <span @click="${this._toggleEditing}">${this.todo.description}</span>
          `}
          <span class="created">Created: ${new Date(this.todo.dateCreated).toLocaleTimeString()}</span>        
        </div>
        <button type="button" @click="${this._removeClicked}" class="remove-button">X</button>
      </div>
    `;
  }

  private _removeClicked(_event: Event) {
    this.dispatchEvent(new CustomEvent('remove-todo', {
      bubbles: true,
      composed: true,
      detail: this.todo
    }));
  }

  private _checkedChanged(_event: Event) {
    this.dispatchEvent(new CustomEvent('toggle-completed', {
      bubbles: true,
      composed: true,
      detail: this.todo
    }));
  }

  private _toggleEditing(_event: Event) {
    this.editing = !this.editing;
  }

  private _updateTodo(event: CustomEvent) {
    this.todo.description = event.detail;
    this.dispatchEvent(new CustomEvent('update-todo', {
      bubbles: true,
      composed: true,
      detail: this.todo
    }));
    this.editing = false;
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'todo-item': TodoItemElement;
  }
}
