import {LitElement, html, css} from 'lit';
import {customElement, property} from 'lit/decorators.js';

import TodoModel from '../models/todo-model';
import './todo-item.js';

@customElement('todo-list')
export class TodoListElement extends LitElement {
  static styles = css`
    :host {
      display: block;
    }
    ul {
      padding: 0;
    }
  `;

  @property({type: Array})
  // TodoModel requires a string in the constructor. This will immediately be replaced with a real TodoModel.
  todos: TodoModel[] = [new TodoModel('')];

  render() {
    return html`
      <ul>
        ${this.todos.map((todo) => html`
          <todo-item
            .todo="${todo}"
            .completed="${todo.completed}"
          >
          </todo-item>`)}
      </ul>
      <div>
        ${this.todos.length ?
          html`${this._remainingItemsText()}` :
          ''
        }
        <button type="button" @click="${this._removeCompleted}">Remove completed</button>
      </div>
    `;
  }

  private _remainingItemsText() {
    const remainingLength = this.todos.filter((todo) => !todo.completed).length;
    return html`<p>${remainingLength === 1 ? '1 item' : `${remainingLength} items`} left</p>`;
  }
  
  private _removeCompleted() {
    this.dispatchEvent(new Event('remove-completed'));
  }
}

declare global {
  interface HTMLElementTagNameMap {
    'todo-list': TodoListElement;
  }
}
