/**
 * @license
 * Copyright 2021 Google LLC
 * SPDX-License-Identifier: BSD-3-Clause
 */

import {TodoLit} from '../todo-lit.js';

import {fixture, html} from '@open-wc/testing';

const assert = chai.assert;

suite('todo-lit', () => {
  test('is defined', () => {
    const el = document.createElement('todo-lit');
    assert.instanceOf(el, TodoLit);
  });

  test('renders with default values', async () => {
    const el = await fixture(html`<todo-lit></todo-lit>`);
    assert.shadowDom.equal(
      el,
      `
      <h1>Hello, World!</h1>
      <button part="button">Click Count: 0</button>
      <slot></slot>
    `
    );
  });

  test('renders with a set name', async () => {
    const el = await fixture(html`<todo-lit name="Test"></todo-lit>`);
    assert.shadowDom.equal(
      el,
      `
      <h1>Hello, Test!</h1>
      <button part="button">Click Count: 0</button>
      <slot></slot>
    `
    );
  });

  test('handles a click', async () => {
    const el = (await fixture(html`<todo-lit></todo-lit>`)) as TodoLit;
    const button = el.shadowRoot!.querySelector('button')!;
    button.click();
    await el.updateComplete;
    assert.shadowDom.equal(
      el,
      `
      <h1>Hello, World!</h1>
      <button part="button">Click Count: 1</button>
      <slot></slot>
    `
    );
  });

  test('styling applied', async () => {
    const el = (await fixture(html`<todo-lit></todo-lit>`)) as TodoLit;
    await el.updateComplete;
    assert.equal(getComputedStyle(el).paddingTop, '16px');
  });
});
